/*
Copyright (C) 2024, PCB Artists OPC Pvt Ltd, all right reserved.

Author:     Pratik Panda
E-mail:     hello@pcbartists.com
Website:    pcbartists.com

The code referencing this license is open source software. Redistribution
and use of the code in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:
    - Redistribution of original and modified source code must retain the 
    above copyright notice, this condition and the following disclaimer.
    This code (or modifications) in source or binary forms may NOT be used
    in a commercial application without obtaining permission from the Author.
    - This software is provided by the copyright holder and contributors "AS IS"
    and any warranties related to this software are DISCLAIMED. The copyright 
    owner or contributors are NOT LIABLE for any damages caused by use of this 
    software.
*/

#ifndef _DBM_FLEX_H_
#define _DBM_FLEX_H_

#include <stdio.h>

// I2C address of the decibel meter (0x48 is default)
#define     DBM_I2C_ADDR        0x48

// Register addresses
#define 	DBM_REG_VERSION		0x00

#define 	DBM_REG_ID3			0x01
#define 	DBM_REG_ID2			0x02
#define 	DBM_REG_ID1			0x03
#define 	DBM_REG_ID0			0x04

#define 	DBM_REG_RSVD		0x05

#define 	DBM_REG_CONTROL		0x06
#define 	DBM_REG_TAVG_HIGH	0x07
#define 	DBM_REG_TAVG_LOW	0x08
#define 	DBM_REG_RESET		0x09

#define 	DBM_REG_DBA_H		10
#define 	DBM_REG_DBA_L		11
#define 	DBM_REG_DBC_H		12
#define 	DBM_REG_DBC_L		13
#define 	DBM_REG_DBZ_H		14
#define 	DBM_REG_DBZ_L		15

#define 	DBM_REG_LAEQF_H		16
#define 	DBM_REG_LAEQF_L		17
#define 	DBM_REG_LCEQF_H		18
#define 	DBM_REG_LCEQF_L		19
#define 	DBM_REG_LZEQF_H		20
#define 	DBM_REG_LZEQF_L		21

#define 	DBM_REG_LAEQS_H		22
#define 	DBM_REG_LAEQS_L		23
#define 	DBM_REG_LCEQS_H		24
#define 	DBM_REG_LCEQS_L		25
#define 	DBM_REG_LZEQS_H		26
#define 	DBM_REG_LZEQS_L		27

#define 	DBM_REG_LAPK_H		28
#define 	DBM_REG_LAPK_L		29
#define 	DBM_REG_LCPK_H		30
#define 	DBM_REG_LCPK_L		31
#define 	DBM_REG_LZPK_H		32
#define 	DBM_REG_LZPK_L		33

#define 	DBM_REG_LAMAX_H		34
#define 	DBM_REG_LAMAX_L		35
#define 	DBM_REG_LCMAX_H		36
#define 	DBM_REG_LCMAX_L		37
#define 	DBM_REG_LZMAX_H		38
#define 	DBM_REG_LZMAX_L		39

#define 	DBM_REG_LAMIN_H		40
#define 	DBM_REG_LAMIN_L		41
#define 	DBM_REG_LCMIN_H		42
#define 	DBM_REG_LCMIN_L		43
#define 	DBM_REG_LZMIN_H		44
#define 	DBM_REG_LZMIN_L		45

#define 	DBM_REG_THR_MIN		46
#define 	DBM_REG_THR_MAX		47

#define 	DBM_REG_T_OVER2		48
#define 	DBM_REG_T_OVER1		49
#define 	DBM_REG_T_OVER0		50
#define 	DBM_REG_T_UNDER2	51
#define 	DBM_REG_T_UNDER1	52
#define 	DBM_REG_T_UNDER0	53

#define 	DBM_REG_HISTORY_SEC	100
#define 	DBM_REG_HISTORY_MIN	160
#define 	DBM_REG_HISTORY_HRS	220

// Combine with an OR '|' to select multiple params
#define     DBM_PARAM_DB        (1<<0)
#define     DBM_PARAM_LEQfast   (1<<1)
#define     DBM_PARAM_LEQslow   (1<<2)
#define     DBM_PARAM_LPK       (1<<3)
#define     DBM_PARAM_LMAXMIN   (1<<4)
#define     DBM_PARAM_TSTATS    (1<<5)

// Combine with an OR '|' to select multiple params
#define     DBM_CLR_INT         (1<<0)
#define     DBM_CLR_MAXMIN      (1<<1)
#define     DBM_CLR_HISTORY     (1<<2)
#define     DBM_CLR_OVERUNDER   (1<<3)
#define     DBM_CLR_LPK         (1<<4)

typedef struct
{
    uint8_t version;
    uint32_t id;
    uint16_t tavg;

    float dBA;
    float dBC;
    float dBZ;

    float LAeqfast;
    float LCeqfast;
    float LZeqfast;

    float LAeqslow;
    float LCeqslow;
    float LZeqslow;

    float LApk;
    float LCpk;
    float LZpk;

    float LAmax;
    float LCmax;
    float LZmax;
    float LAmin;
    float LCmin;
    float LZmin;

    uint8_t threshold_min;
    uint8_t threshold_max;

    uint32_t seconds_over;
    uint32_t seconds_under;
} dbm_flex;

esp_err_t dbmeter_get_version_id (i2c_port_t i2c, dbm_flex *dbm);
esp_err_t dbmeter_update_params (i2c_port_t i2c, dbm_flex *dbm, uint8_t params);
esp_err_t dbmeter_set_tavg (i2c_port_t i2c, dbm_flex *dbm);
esp_err_t dbmeter_set_sleep (i2c_port_t i2c, uint8_t activate_sleep);
esp_err_t dbmeter_clear_data (i2c_port_t i2c, uint8_t params);
esp_err_t dbmeter_read_history_sec (i2c_port_t i2c, uint8_t *history);
esp_err_t dbmeter_read_history_min (i2c_port_t i2c, uint8_t *history);
esp_err_t dbmeter_read_history_hrs (i2c_port_t i2c, uint8_t *history);

#endif
